package com.isu.gaswellwatch.modbus.data;

import cn.hutool.core.map.MapUtil;
import org.apache.commons.io.IOUtils;
import org.springframework.util.ResourceUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Map;
import java.util.Objects;

/**
 * 数据持久化处理器
 *
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/23 11:53
 */
public interface PersistenceHandler {
    String KNPCV1_MODBUS_TYPE = "knpc";
    String ETC_MODBUS_TYPE = "etc";
    String SCSS_MODBUS_TYPE = "scss";


    String DEVICE_INFO_CACHE = "info:device:";
    String DEVICE_DATA_CACHE = "data:device:";

    void createTable(String tableName, Long deviceId);

    void insert(String tableName, String cacheKey);

    default void setValue(PreparedStatement ps, Map<String, Object> row, int index, String key, int sqlType) throws SQLException {
        String value = MapUtil.getStr(row, key);
        if (Objects.isNull(value)) {
            ps.setNull(index, sqlType);
        } else {
            ps.setObject(index, value);
        }
    }

    static String getResource(String classPath) {
        try {
            File file = ResourceUtils.getFile("classpath:" + classPath);
            return IOUtils.toString(new FileInputStream(file), StandardCharsets.UTF_8);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }


}
