package com.isu.gaswellwatch.modbus.data.impl;

import com.isu.gaswellwatch.config.SnowflakeConfig;
import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.stereotype.Component;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Map;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/24 17:33
 */
@Component(PersistenceHandler.ETC_MODBUS_TYPE)
public class EtcPersistenceHandler implements PersistenceHandler {

    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource
    private SnowflakeConfig snowflakeConfig;
    @Resource(name = "stringRedisTemplate")
    private RedisTemplate redisTemplate;

    @Override
    public void createTable(String tableName, Long deviceId) {
        String createTableSQL = PersistenceHandler.getResource("sql/CREATE_ETC.sql");
        createTableSQL = StringUtils.replace(createTableSQL, "$TableName$", tableName);
        createTableSQL = StringUtils.replace(createTableSQL, "$DeviceId$", String.valueOf(deviceId));
        this.jdbcTemplate.execute(createTableSQL);
    }

    @Override
    public void insert(String tableName, String cacheKey) {
        String insertTableSQL = PersistenceHandler.getResource("sql/INSERT_ETC.sql");
        insertTableSQL = StringUtils.replace(insertTableSQL, "$TableName$", tableName);

        Map<String, Object> row = this.redisTemplate.opsForHash().entries(cacheKey);

        this.jdbcTemplate.execute(insertTableSQL, new PreparedStatementCallback<Object>() {
            @Override
            public Object doInPreparedStatement(PreparedStatement ps) throws SQLException {
                ps.setLong(1, EtcPersistenceHandler.this.snowflakeConfig.snowflakeId());
                EtcPersistenceHandler.this.setValue(ps, row, 2, "deviceId", Types.BIGINT);
                EtcPersistenceHandler.this.setValue(ps, row, 3, "collectionTime", Types.TIMESTAMP);
                EtcPersistenceHandler.this.setValue(ps, row, 4, "receiveTime", Types.TIMESTAMP);
                EtcPersistenceHandler.this.setValue(ps, row, 5, "deviceTime", Types.TIMESTAMP);
                EtcPersistenceHandler.this.setValue(ps, row, 6, "runMode", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 7, "wellStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 8, "plugStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 9, "statusOpenTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 10, "statusCloseTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 11, "oilPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 12, "casPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 13, "prePressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 14, "pipePressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 15, "liquidLevel", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 16, "temperature", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 17, "humidity", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 18, "opmode", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 19, "timer1", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 20, "timer2", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 21, "timer1Open", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 22, "timer1Close", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 23, "timer2Open", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 24, "timer2Close", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 25, "timingOpen", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 26, "timingClose", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 27, "timingCelay", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 28, "presource", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 29, "pressureOpen", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 30, "pressureClose", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 31, "triggerType", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 32, "stabilityTime", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 33, "maxOpenWell", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 34, "maxCloseWell", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 35, "minOpenWell", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 36, "minCloseWell", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 37, "presproTect", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 38, "presproSource", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 39, "openWellLimitMax", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 40, "openWellLimitMin", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, row, 41, "plugInitStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 42, "plugSustainTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 43, "plugCloseTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 44, "tpInitStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 45, "tpOpenSource", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 46, "tpOpenTrigger", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 47, "tpOpenPressure", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 48, "tpOpenTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 49, "tpCloseSource", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 50, "tpCloseTrigger", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 51, "tpClosePressure", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, row, 52, "tpCloseTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, row, 53, "tpStabilityTime", Types.INTEGER);
                return ps.executeUpdate();
            }
        });
    }
}
